@@ -32,16 +32,19 @@ from beartype._data.hint.pep.sign.datapepsigns import (
     HintSignFinal,
     HintSignGeneric,
     # HintSignIO,
+    HintSignLiteralString,
     HintSignNewType,
     HintSignNone,
     HintSignNumpyArray,
     HintSignPanderaAny,
+    HintSignSelf,
     # HintSignTextIO,
     HintSignType,
     HintSignTypeGuard,
     HintSignTypeVar,
     HintSignTypedDict,
 )
+from beartype._data.datatyping import TypeStack
 from beartype._util.cache.utilcachecall import callable_cached
 from beartype._util.hint.nonpep.mod.utilmodnumpy import (
     reduce_hint_numpy_ndarray)
@@ -63,24 +66,40 @@ from beartype._util.hint.pep.proposal.utilpep589 import reduce_hint_pep589
 from beartype._util.hint.pep.proposal.utilpep591 import reduce_hint_pep591
 from beartype._util.hint.pep.proposal.utilpep593 import reduce_hint_pep593
 from beartype._util.hint.pep.proposal.utilpep647 import reduce_hint_pep647
+from beartype._util.hint.pep.proposal.utilpep673 import reduce_hint_pep673
+from beartype._util.hint.pep.proposal.utilpep675 import reduce_hint_pep675
 from beartype._util.hint.pep.utilpepget import get_hint_pep_sign_or_none
 from beartype._util.hint.pep.utilpepreduce import reduce_hint_pep_unsigned
 from collections.abc import Callable
 
 # ....................{ REDUCERS                           }....................
-@callable_cached
 def reduce_hint(
+    # Mandatory parameters.
     hint: Any,
     conf: BeartypeConf,
-    arg_name: Optional[str],
-    exception_prefix: str,
+
+    # Optional parameters.
+    cls_stack: TypeStack = None,
+    arg_name: Optional[str] = None,
+    exception_prefix: str = '',
 ) -> object:
     '''
     Lower-level type hint reduced (i.e., converted) from the passed higher-level
     type hint if this hint is reducible *or* this hint as is otherwise (i.e., if
     this hint is irreducible).
 
-    This reducer is memoized for efficiency.
+    This reducer *cannot* be meaningfully memoized, since multiple passed
+    parameters (e.g., ``arg_name``, ``cls_stack``) are typically isolated to a
+    handful of callables across the codebase currently being decorated by
+    :mod:`beartype`. Memoizing this reducer would needlessly consume space and
+    time. To improve efficiency, this reducer is instead implemented in terms of
+    two lower-level private reducers:
+
+    * The memoized :func:`._reduce_hint_cached` reducer, responsible for
+      efficiently reducing *most* (but not all) type hints.
+    * The unmemoized :func:`._reduce_hint_uncached` reducer, responsible for
+      inefficiently reducing the small subset of type hints contextually
+      requiring these problematic parameters.
 
     Parameters
     ----------
@@ -89,32 +108,88 @@ def reduce_hint(
     conf : BeartypeConf
         **Beartype configuration** (i.e., self-caching dataclass encapsulating
         all settings configuring type-checking for the passed object).
-    arg_name : Optional[str]
+    cls_stack : TypeStack, optional
+        **Type stack** (i.e., either a tuple of the one or more
+        :func:`beartype.beartype`-decorated classes lexically containing the
+        class variable or method annotated by this hint *or* :data:`None`).
+        Defaults to :data:`None`.
+    arg_name : Optional[str], optional
         Either:
 
         * If this hint annotates a parameter of some callable, the name of that
           parameter.
         * If this hint annotates the return of some callable, ``"return"``.
         * Else, :data:`None`.
-    exception_prefix : str
-        Human-readable label prefixing the representation of this object in the
-        exception message.
+
+        Defaults to :data:`None`.
+    exception_prefix : str, optional
+        Substring prefixing exception messages raised by this function. Defaults
+        to the empty string.
 
     Returns
     ----------
     object
         Either:
 
-        * If the passed higher-level type hint is reducible, a lower-level type
-          hint reduced (i.e., converted, extracted) from this hint.
+        * If the passed hint is reducible, another hint reduced from this hint.
         * Else, this hint as is unmodified.
+    '''
 
-    Raises
+    # This possibly contextual hint inefficiently reduced to another hint.
+    #
+    # Note that we intentionally reduce lower-level contextual hints *BEFORE*
+    # higher-level context-free hints. In theory, order of reduction *SHOULD* be
+    # insignificant; in practice, we suspect unforeseen and unpredictable
+    # interactions between these two reductions. To reduce the likelihood of
+    # fire-breathing dragons, we reduce lower-level hints first.
+    hint = _reduce_hint_uncached(
+        hint=hint,
+        conf=conf,
+        arg_name=arg_name,
+        cls_stack=cls_stack,
+        exception_prefix=exception_prefix,
+    )
+
+    # This possibly context-free hint efficiently reduced to another hint.
+    hint = _reduce_hint_cached(hint, conf, exception_prefix)
+
+    # Return this possibly reduced hint.
+    return hint
+
+# ....................{ PRIVATE ~ reducers                 }....................
+@callable_cached
+def _reduce_hint_cached(
+    hint: Any,
+    conf: BeartypeConf,
+    exception_prefix: str,
+) -> object:
+    '''
+    Lower-level **context-free type hint** (i.e., type hint *not* contextually
+    dependent on the kind of class, attribute, callable parameter, or callable
+    return annotated by this hint) efficiently reduced (i.e., converted) from
+    the passed higher-level context-free type hint if this hint is reducible
+    *or* this hint as is otherwise (i.e., if this hint is irreducible).
+
+    This reducer is memoized for efficiency. Thankfully, this reducer is
+    responsible for reducing *most* (but not all) type hints.
+
+    Parameters
     ----------
-    BeartypeDecorHintNonpepNumpyException
-        See the
-        :func:`beartype._util.hint.nonpep.mod.utilmodnumpy.reduce_hint_numpy_ndarray`
-        function for further details.
+    hint : Any
+        Type hint to be possibly reduced.
+    conf : BeartypeConf
+        **Beartype configuration** (i.e., self-caching dataclass encapsulating
+        all settings configuring type-checking for the passed object).
+    exception_prefix : str
+        Substring prefixing exception messages raised by this function.
+
+    Returns
+    ----------
+    object
+        Either:
+
+        * If the passed hint is reducible, another hint reduced from this hint.
+        * Else, this hint as is unmodified.
     '''
 
     # Sign uniquely identifying this hint if this hint is identifiable *OR*
@@ -124,7 +199,7 @@ def reduce_hint(
     # Callable reducing this hint if a callable reducing hints of this sign was
     # previously registered *OR* "None" otherwise (i.e., if *NO* such callable
     # was registered, in which case this hint is preserved as is).
-    hint_reducer = _HINT_SIGN_TO_REDUCER.get(hint_sign)
+    hint_reducer = _HINT_SIGN_TO_REDUCE_HINT_CACHED.get(hint_sign)
 
     # If a callable reducing hints of this sign was previously registered,
     # reduce this hint to another hint via this callable.
@@ -132,6 +207,79 @@ def reduce_hint(
         hint = hint_reducer(  # type: ignore[call-arg]
             hint=hint,  # pyright: ignore[reportGeneralTypeIssues]
             conf=conf,
+            exception_prefix=exception_prefix,
+        )
+    # Else, *NO* such callable was registered. Preserve this hint as is, you!
+
+    # Return this possibly reduced hint.
+    return hint
+
+
+def _reduce_hint_uncached(
+    hint: Any,
+    conf: BeartypeConf,
+    cls_stack: TypeStack,
+    arg_name: Optional[str],
+    exception_prefix: str,
+) -> object:
+    '''
+    Lower-level **contextual type hint** (i.e., type hint contextually dependent
+    on the kind of class, attribute, callable parameter, or callable return
+    annotated by this hint) inefficiently reduced (i.e., converted) from the
+    passed higher-level context-free type hint if this hint is reducible *or*
+    this hint as is otherwise (i.e., if this hint is irreducible).
+
+    This reducer *cannot* be meaningfully memoized, since multiple passed
+    parameters (e.g., ``arg_name``, ``cls_stack``) are typically isolated to a
+    handful of callables across the codebase currently being decorated by
+    :mod:`beartype`. Thankfully, this reducer is responsible for reducing only a
+    small subset of type hints requiring these problematic parameters.
+
+    Parameters
+    ----------
+    hint : Any
+        Type hint to be possibly reduced.
+    conf : BeartypeConf
+        **Beartype configuration** (i.e., self-caching dataclass encapsulating
+        all settings configuring type-checking for the passed object).
+    cls_stack : TypeStack
+        **Type stack** (i.e., either tuple of zero or more arbitrary types *or*
+        :data:`None`). See also the :func:`.beartype_object` decorator.
+    arg_name : Optional[str]
+        Either:
+
+        * If this hint annotates a parameter of some callable, the name of that
+          parameter.
+        * If this hint annotates the return of some callable, ``"return"``.
+        * Else, :data:`None`.
+    exception_prefix : str
+        Substring prefixing exception messages raised by this function.
+
+    Returns
+    ----------
+    object
+        Either:
+
+        * If the passed hint is reducible, another hint reduced from this hint.
+        * Else, this hint as is unmodified.
+    '''
+
+    # Sign uniquely identifying this hint if this hint is identifiable *OR*
+    # "None" otherwise.
+    hint_sign = get_hint_pep_sign_or_none(hint)
+
+    # Callable reducing this hint if a callable reducing hints of this sign was
+    # previously registered *OR* "None" otherwise (i.e., if *NO* such callable
+    # was registered, in which case this hint is preserved as is).
+    hint_reducer = _HINT_SIGN_TO_REDUCE_HINT_UNCACHED.get(hint_sign)
+
+    # If a callable reducing hints of this sign was previously registered,
+    # reduce this hint to another hint via this callable.
+    if hint_reducer is not None:
+        hint = hint_reducer(  # type: ignore[call-arg]
+            hint=hint,  # pyright: ignore[reportGeneralTypeIssues]
+            conf=conf,
+            cls_stack=cls_stack,
             arg_name=arg_name,
             exception_prefix=exception_prefix,
         )
@@ -140,7 +288,15 @@ def reduce_hint(
     # Return this possibly reduced hint.
     return hint
 
-# ....................{ PRIVATE ~ globals                  }....................
+# ....................{ PRIVATE ~ hints                    }....................
+_DictReducer = Dict[Optional[HintSign], Callable]
+'''
+PEP-compliant type hint matching a **reducer dictionary** (i.e., dictionary
+mapping from each sign uniquely identifying various type hints to a callable
+reducing those higher- to lower-level hints).
+'''
+
+# ....................{ PRIVATE ~ dicts                    }....................
 #FIXME: After dropping Python 3.7 support:
 #* Replace "Callable[[object, str], object]" here with a callback protocol:
 #      https://mypy.readthedocs.io/en/stable/protocols.html#callback-protocols
@@ -148,7 +304,7 @@ def reduce_hint(
 #  these callables with keyword arguments above! Chaos ensues.
 #* Remove the "# type: ignore[call-arg]" pragmas above, which are horrible.
 #* Remove the "# type: ignore[dict-item]" pragmas above, which are horrible.
-_HINT_SIGN_TO_REDUCER: Dict[Optional[HintSign], Callable] = {
+_HINT_SIGN_TO_REDUCE_HINT_CACHED: _DictReducer = {
     # ..................{ NON-PEP                            }..................
     # If this hint is identified by *NO* sign, this hint is either an
     # isinstanceable type *OR* a hint unrecognized by beartype. In either case,
@@ -229,9 +385,7 @@ _HINT_SIGN_TO_REDUCER: Dict[Optional[HintSign], Callable] = {
     HintSignTypedDict: reduce_hint_pep589,
 
     # ..................{ PEP 591                            }..................
-    #FIXME: Remove *AFTER* deeply type-checking typed dictionaries. For now,
-    #shallowly type-checking such hints by reduction to untyped dictionaries
-    #remains the sanest temporary work-around.
+    #FIXME: Remove *AFTER* deeply type-checking final type hints.
 
     # If this hint is a PEP 591-compliant "typing.Final[...]" type hint,
     # silently reduce this hint to its subscripted argument (e.g., from
@@ -244,13 +398,19 @@ _HINT_SIGN_TO_REDUCER: Dict[Optional[HintSign], Callable] = {
     # lower-level hint it annotates.
     HintSignAnnotated: reduce_hint_pep593,
 
-    # ..................{ PEP 647                            }..................
-    # If this hint is a PEP 647-compliant "typing.TypeGuard[...]" type hint,
-    # either:
-    # * If this hint annotates the return of some callable, reduce this hint to
-    #   the standard "bool" type.
-    # * Else, raise an exception.
-    HintSignTypeGuard: reduce_hint_pep647,
+    # ..................{ PEP 675                            }..................
+    #FIXME: Remove *AFTER* deeply type-checking literal strings. Note that doing
+    #so will prove extremely non-trivial or possibly even infeasible, suggesting
+    #we will probably *NEVER* deeply type-check literal strings. It's *NOT*
+    #simply a matter of efficiently parsing ASTs at runtime; it's that as well
+    #as correctly transitively inferring literal strings across operations and
+    #calls, which effectively requires parsing the entire codebase and
+    #constructing an in-memory graph of all type relations. See also:
+    #    https://peps.python.org/pep-0675/#inferring-literalstring
+
+    # If this hint is a PEP 675-compliant "typing.LiteralString" type hint,
+    # reduce this hint to the standard "str" type.
+    HintSignLiteralString: reduce_hint_pep675,
 
     # ..................{ NON-PEP ~ numpy                    }..................
     # If this hint is a PEP-noncompliant typed NumPy array (e.g.,
@@ -266,15 +426,15 @@ _HINT_SIGN_TO_REDUCER: Dict[Optional[HintSign], Callable] = {
 }
 '''
 Dictionary mapping from each sign uniquely identifying PEP-compliant type hints
-to that sign's **reducer** (i.e., callable reducing those higher-level hints to
-lower-level type hints).
+to that sign's **cached reducer** (i.e., callable efficiently memoized by the
+:func:`.callable_cached` decorator reducing those higher- to lower-level hints).
 
 Each value of this dictionary should be a valid reducer, defined as a function
 with signature resembling:
 
 .. code-block:: python
 
-   def reduce_pep{pep_number}_hint(
+   def reduce_hint_pep{pep_number}(
        hint: object,
        conf: BeartypeConf,
        arg_name: Optional[str],
@@ -291,13 +451,41 @@ Note that:
   expected sign. By design, a reducer is only ever passed a type hint of the
   expected sign.
 * Reducers should *not* be memoized (e.g., by the
-  :func:`beartype._util.cache.utilcachecall.callable_cached` decorator). Since
-  the higher-level :func:`.reduce_hint` function that is the sole entry point to
-  calling all lower-level reducers is itself memoized, reducers themselves do
-  neither require nor benefit from memoization. Moreover, even if they did
-  either require or benefit from memoization, they couldn't be -- at least, not
+  :func:`.callable_cached` decorator). Since the higher-level
+  :func:`.reduce_hint` function that is the sole entry point to calling all
+  lower-level reducers is itself memoized, reducers themselves do neither
+  require nor benefit from memoization. Moreover, even if they did either
+  require or benefit from memoization, they couldn't be -- at least, not
   directly. Why? Because :func:`.reduce_hint` necessarily passes keyword
   arguments to all reducers. But memoized functions *cannot* receive keyword
   arguments (without destroying efficiency and thus the entire impetus for
   memoization). Ergo, reducers *cannot* be memoized.
 '''
+
+
+_HINT_SIGN_TO_REDUCE_HINT_UNCACHED: _DictReducer = {
+    # ..................{ PEP 647                            }..................
+    # If this hint is a PEP 647-compliant "typing.TypeGuard[...]" type hint,
+    # either:
+    # * If this hint annotates the return of some callable, reduce this hint to
+    #   the standard "bool" type.
+    # * Else, raise an exception.
+    HintSignTypeGuard: reduce_hint_pep647,
+
+    # ..................{ PEP 673                            }..................
+    # If this hint is a PEP 673-compliant "typing.Self" type hint, either:
+    # * If @beartype is currently decorating a class, reduce this hint to the
+    #   most deeply nested class on the passed type stack.
+    # * Else, raise an exception.
+    HintSignSelf: reduce_hint_pep673,
+}
+'''
+Dictionary mapping from each sign uniquely identifying various type hints to
+that sign's **cached reducer** (i.e., callable efficiently memoized by the
+:func:`.callable_cached` decorator reducing those higher- to lower-level hints).
+
+See Also
+----------
+:data:`._HINT_SIGN_TO_REDUCE_HINT_CACHED`
+    Further details.
+'''
