@@ -16,7 +16,9 @@ from beartype.typing import (
     Dict,
     FrozenSet,
     Set,
+    # Union,
 )
+from beartype._data.datatyping import HintSignTrie
 from beartype._data.hint.pep.sign import datapepsigns
 from beartype._data.hint.pep.sign.datapepsigncls import HintSign
 from beartype._data.hint.pep.sign.datapepsigns import (
@@ -29,6 +31,7 @@ from beartype._data.hint.pep.sign.datapepsigns import (
     HintSignAsyncIterator,
     HintSignAsyncGenerator,
     HintSignAwaitable,
+    HintSignBinaryIO,
     HintSignByteString,
     HintSignCallable,
     HintSignChainMap,
@@ -48,6 +51,7 @@ from beartype._data.hint.pep.sign.datapepsigns import (
     HintSignGenerator,
     # HintSignGeneric,
     # HintSignHashable,
+    HintSignIO,
     HintSignItemsView,
     HintSignIterable,
     HintSignIterator,
@@ -66,6 +70,7 @@ from beartype._data.hint.pep.sign.datapepsigns import (
     HintSignNumpyArray,
     # HintSignOptional,
     HintSignOrderedDict,
+    HintSignPanderaAny,
     HintSignParamSpec,
     # HintSignParamSpecArgs,
     # HintSignProtocol,
@@ -74,6 +79,7 @@ from beartype._data.hint.pep.sign.datapepsigns import (
     HintSignSet,
     # HintSignSized,
     HintSignPattern,
+    HintSignTextIO,
     HintSignTuple,
     HintSignType,
     HintSignTypeVar,
@@ -113,24 +119,31 @@ HINT_REPR_PREFIX_ARGS_0_OR_MORE_TO_SIGN: Dict[str, HintSign] = {
     # semantics rather than *NO* such semantics, which would then substantially
     # break and complicate dynamic code generation for no benefit whatsoever.
     'None': HintSignNone,
+
+    # PEP 484-compliant abstract base classes (ABCs) requiring non-standard and
+    # non-trivial type-checking. Although most types are trivially type-checked
+    # by the isinstance() builtin, these types break the mold in various ways.
+    "<class 'typing.BinaryIO'>": HintSignBinaryIO,
+    "<class 'typing.IO'>": HintSignIO,
+    "<class 'typing.TextIO'>": HintSignTextIO,
 }
 '''
 Dictionary mapping from the **possibly unsubscripted PEP-compliant type hint
 representation prefix** (i.e., unsubscripted prefix of the machine-readable
 strings returned by the :func:`repr` builtin for PEP-compliant type hints
 permissible in both subscripted and unsubscripted forms) of each hint uniquely
-identifiable by that representations to its identifying sign.
+identifiable by that representation to its identifying sign.
 
 Notably, this dictionary maps from the representation prefixes of:
 
-* *All* :pep:`484`-compliant type hints. Whereas *all* :pep:`585`-compliant
-  type hints (e.g., ``list[str]``) are necessarily subscripted and thus omitted
-  from this dictionary, *all* :pep:`484`-compliant type hints support at least
+* *All* :pep:`484`-compliant type hints. Whereas *all* :pep:`585`-compliant type
+  hints (e.g., ``list[str]``) are necessarily subscripted and thus omitted from
+  this dictionary, *all* :pep:`484`-compliant type hints support at least
   unsubscripted form and most :pep:`484`-compliant type hints support
   subscription as well. Moreover, the unsubscripted forms of most
   :pep:`484`-compliant type hints convey deep semantics and thus require
-  detection as PEP-compliant (e.g., ``typing.List``, requiring detection and
-  reduction to ``list``).
+  detection as PEP-compliant (e.g., :obj:`typing.List`, requiring detection and
+  reduction to :class:`list`).
 '''
 
 
@@ -220,6 +233,28 @@ Notably, this dictionary maps from the representation prefixes of:
   detected as PEP-compliant.
 '''
 
+# ....................{ MAPPINGS ~ repr : trie             }....................
+# The majority of this trie is defined by explicit key-value pairs here.
+HINT_REPR_PREFIX_TRIE_ARGS_0_OR_MORE_TO_SIGN: HintSignTrie = {
+    # ..................{ NON-PEP ~ lib : pandera            }..................
+    # All PEP-noncompliant "pandera.typing" type hints are permissible in
+    # both subscripted and unsubscripted forms.
+    'pandera': {
+        'typing': HintSignPanderaAny,
+    }
+}
+'''
+**Sign trie** (i.e., dictionary-of-dictionaries tree data structure enabling
+efficient mapping from the machine-readable representations of type hints
+created by an arbitrary number of type hint factories defined by an external
+third-party package to their identifying sign) from the **possibly unsubscripted
+PEP-compliant type hint representation prefix** (i.e., unsubscripted prefix of
+the machine-readable strings returned by the :func:`repr` builtin for
+PEP-compliant type hints permissible in both subscripted and unsubscripted
+forms) of each hint uniquely identifiable by that representation to its
+identifying sign.
+'''
+
 # ....................{ MAPPINGS ~ type                    }....................
 # The majority of this dictionary is initialized with automated inspection
 # below in the _init() function. The *ONLY* key-value pairs explicitly defined
@@ -323,6 +358,10 @@ HINTS_REPR_IGNORABLE_SHALLOW: FrozenSet[str] = {  # type: ignore[assignment]
     #     def muh_union_printer(muh_union: UnionType) -> None: print(muh_union)
     #
     # Ergo, we intentionally omit this type from consideration here.
+
+    # ....................{ NON-PEP                        }....................
+    # Machine-readable representations of shallowly ignorable type hints
+    # published by PEP-noncompliant third-party type hints, including...
 }
 '''
 Frozen set of all **shallowly ignorable PEP-compliant type hint
