@@ -39,11 +39,15 @@ from beartype._check.expr._exprsnip import (
     PEP_CODE_HINT_FORWARDREF_UNQUALIFIED_PLACEHOLDER_SUFFIX,
 )
 from beartype._check.util.checkutilmake import make_func_signature
+from beartype._check.checkcall import BeartypeCall
+from beartype._data.datakind import (
+    ARG_NAME_RETURN,
+    ARG_NAME_RETURN_REPR,
+)
 from beartype._decor._cache.cachetype import (
     bear_typistry,
     register_typistry_forwardref,
 )
-from beartype._check.checkcall import BeartypeCall
 from beartype._decor._wrap.wrapsnip import (
     CODE_INIT_ARGS_LEN,
     CODE_PITH_ROOT_PARAM_NAME_PLACEHOLDER,
@@ -65,6 +69,7 @@ from beartype._util.func.arg.utilfuncargiter import (
     ArgKind,
     iter_func_args,
 )
+from beartype._util.hint.utilhinttest import is_hint_needs_cls_stack
 from beartype._util.hint.pep.proposal.pep484585.utilpep484585func import (
     reduce_hint_pep484585_func_return)
 from beartype._util.hint.pep.proposal.pep484585.utilpep484585ref import (
@@ -72,11 +77,11 @@ from beartype._util.hint.pep.proposal.pep484585.utilpep484585ref import (
 from beartype._check.conv.convsanify import sanify_hint_root_func
 from beartype._util.hint.utilhinttest import is_hint_ignorable
 from beartype._util.kind.utilkinddict import update_mapping
-from beartype._util.text.utiltextlabel import (
-    prefix_callable_decorated_arg,
-    prefix_callable_decorated_return,
-)
 from beartype._util.text.utiltextmunge import replace_str_substrs
+from beartype._util.text.utiltextprefix import (
+    prefix_beartypeable_arg,
+    prefix_beartypeable_return,
+)
 from beartype._util.utilobject import SENTINEL
 from collections.abc import (
     Callable,
@@ -257,14 +262,6 @@ Frozen set of all **positional parameter kinds** (i.e.,
 either may *or* must be passed positionally).
 '''
 
-
-_RETURN_REPR = repr('return')
-'''
-Object representation of the magic string implying a return value in various
-Python objects (e.g., the ``__annotations__`` dunder dictionary of annotated
-callables).
-'''
-
 # ....................{ PRIVATE ~ args                     }....................
 def _code_check_args(bear_call: BeartypeCall) -> str:
     '''
@@ -314,7 +311,7 @@ def _code_check_args(bear_call: BeartypeCall) -> str:
         len(bear_call.func_arg_name_to_hint) == 1 and
         # That type hint annotates that callable's return rather than a
         # parameter accepted by that callable...
-        'return' in bear_call.func_arg_name_to_hint
+        ARG_NAME_RETURN in bear_call.func_arg_name_to_hint
     ):
         return ''
     # Else, one or more callable parameters are annotated.
@@ -329,10 +326,14 @@ def _code_check_args(bear_call: BeartypeCall) -> str:
     is_args_positional = False
 
     # ..................{ LOCALS ~ hint                      }..................
-    # Type hint annotating this parameter if any *OR* "_PARAM_HINT_EMPTY"
+    # Type hint annotating the current parameter if any *OR* "_PARAM_HINT_EMPTY"
     # otherwise (i.e., if this parameter is unannotated).
     hint = None
 
+    # This type hint sanitized into a possibly different type hint more readily
+    # consumable by @beartype's code generator.
+    hint_sane = None
+
     # ..................{ GENERATE                           }..................
     #FIXME: Locally remove the "arg_index" local variable (and thus avoid
     #calling the enumerate() builtin here) AFTER* refactoring @beartype to
@@ -349,7 +350,7 @@ def _code_check_args(bear_call: BeartypeCall) -> str:
         # renders the latter mostly useless for our purposes.
         func=bear_call.func_wrappee_wrappee,
         func_codeobj=bear_call.func_wrappee_wrappee_codeobj,
-        is_unwrapping=False,
+        is_unwrap=False,
     )):
         # Kind and name of this parameter.
         arg_kind: ArgKind = arg_meta[ARG_META_INDEX_KIND]  # type: ignore[assignment]
@@ -381,15 +382,10 @@ def _code_check_args(bear_call: BeartypeCall) -> str:
                 continue
             # Else, this parameter is non-ignorable.
 
-            # Sanitize this hint to either:
-            # * If this hint is PEP-noncompliant, the PEP-compliant type hint
-            #   converted from this PEP-noncompliant type hint.
-            # * If this hint is both PEP-compliant and supported, this hint as
-            #   is.
-            # * Else, raise an exception.
-            #
-            # Do this first *BEFORE* passing this hint to any further callables.
-            hint = sanify_hint_root_func(
+            # Sanitize this hint into a possibly different type hint more
+            # readily consumable by @beartype's code generator *BEFORE* passing
+            # this hint to any further callables.
+            hint_sane = sanify_hint_root_func(
                 hint=hint, arg_name=arg_name, bear_call=bear_call)
 
             # If this hint is ignorable, continue to the next parameter.
@@ -397,7 +393,7 @@ def _code_check_args(bear_call: BeartypeCall) -> str:
             # Note that this is intentionally tested *AFTER* this hint has been
             # coerced into a PEP-compliant type hint to implicitly ignore
             # PEP-noncompliant type hints as well (e.g., "(object, int, str)").
-            if is_hint_ignorable(hint):
+            if is_hint_ignorable(hint_sane):
                 # print(f'Ignoring {bear_call.func_name} parameter {arg_name} hint {repr(hint)}...')
                 continue
             # Else, this hint is unignorable.
@@ -436,13 +432,27 @@ def _code_check_args(bear_call: BeartypeCall) -> str:
             # Else, this kind of parameter is supported. Ergo, this code is
             # non-"None".
 
+            # Type stack if required by this hint *OR* "None" otherwise. See the
+            # is_hint_needs_cls_stack() tester for further discussion.
+            #
+            # Note that the original unsanitized "hint" (e.g., "typing.Self")
+            # rather than the new sanitized "hint_sane" (e.g., the class
+            # currently being decorated by @beartype) is passed to that tester.
+            # Why? Because the latter may already have been reduced above to a
+            # different and seemingly innocuous type hint that does *NOT* appear
+            # to require a type stack but actually does. Only the original
+            # unsanitized "hint" can tell the truth.
+            cls_stack = (
+                 bear_call.cls_stack if is_hint_needs_cls_stack(hint) else None)
+            # print(f'arg "{arg_name}" hint {repr(hint)} cls_stack: {repr(cls_stack)}')
+
             # Generate a memoized parameter-agnostic code snippet type-checking
             # any parameter or return value with an arbitrary name.
             (
                 code_param_check_pith,
                 func_wrapper_scope,
                 hint_forwardrefs_class_basename,
-            ) = make_func_wrapper_code(hint, bear_call.conf)
+            ) = make_func_wrapper_code(hint_sane, bear_call.conf, cls_stack)
 
             # Merge the local scope required to check this parameter into the
             # local scope currently required by the current wrapper function.
@@ -472,9 +482,9 @@ def _code_check_args(bear_call: BeartypeCall) -> str:
                 #FIXME: Embed the kind of parameter as well (e.g.,
                 #"positional-only", "keyword-only", "variadic positional"),
                 #ideally by improving the existing
-                #prefix_callable_decorated_arg() function to introspect this
+                #prefix_beartypeable_arg() function to introspect this
                 #kind from that callable's code object.
-                target_str=prefix_callable_decorated_arg(
+                target_str=prefix_beartypeable_arg(
                     func=bear_call.func_wrappee, arg_name=arg_name),
             )
 
@@ -532,7 +542,7 @@ def _code_check_return(bear_call: BeartypeCall) -> str:
     # Note that "None" is a semantically meaningful PEP 484-compliant type hint
     # equivalent to "type(None)". Ergo, we *MUST* explicitly distinguish
     # between that type hint and an unannotated return.
-    hint = bear_call.func_arg_name_to_hint_get('return', SENTINEL)
+    hint = bear_call.func_arg_name_to_hint_get(ARG_NAME_RETURN, SENTINEL)
 
     # If this return is unannotated, silently reduce to a noop.
     if hint is SENTINEL:
@@ -574,12 +584,26 @@ def _code_check_return(bear_call: BeartypeCall) -> str:
             # * Else, raise an exception.
             #
             # Do this first *BEFORE* passing this hint to any further callables.
-            hint = sanify_hint_root_func(
-                hint=hint, arg_name='return', bear_call=bear_call)
+            hint_sane = sanify_hint_root_func(
+                hint=hint, arg_name=ARG_NAME_RETURN, bear_call=bear_call)
 
             # If this PEP-compliant hint is unignorable, generate and return a
             # snippet type-checking this return against this hint.
-            if not is_hint_ignorable(hint):
+            if not is_hint_ignorable(hint_sane):
+                # Type stack if required by this hint *OR* "None" otherwise. See
+                # the is_hint_needs_cls_stack() tester for further discussion.
+                #
+                # Note that the original unsanitized "hint" (e.g.,
+                # "typing.Self") rather than the new sanitized "hint_sane"
+                # (e.g., the class currently being decorated by @beartype) is
+                # passed to that tester. See _code_check_args() for details.
+                cls_stack = (
+                    bear_call.cls_stack
+                    if is_hint_needs_cls_stack(hint) else
+                    None
+                )
+                # print(f'return hint {repr(hint)} cls_stack: {repr(cls_stack)}')
+
                 # Empty tuple, passed below to satisfy the
                 # _unmemoize_func_wrapper_code() API.
                 hint_forwardrefs_class_basename = ()
@@ -590,7 +614,7 @@ def _code_check_return(bear_call: BeartypeCall) -> str:
                     code_return_check_pith,
                     func_wrapper_scope,
                     hint_forwardrefs_class_basename,
-                ) = make_func_wrapper_code(hint, bear_call.conf)  # type: ignore[assignment]
+                ) = make_func_wrapper_code(hint_sane, bear_call.conf, cls_stack)  # type: ignore[assignment]
 
                 # Merge the local scope required to type-check this return into
                 # the local scope currently required by the current wrapper
@@ -602,11 +626,25 @@ def _code_check_return(bear_call: BeartypeCall) -> str:
                 code_return_check_pith_unmemoized = _unmemoize_func_wrapper_code(
                     bear_call=bear_call,
                     func_wrapper_code=code_return_check_pith,
-                    pith_repr=_RETURN_REPR,
+                    pith_repr=ARG_NAME_RETURN_REPR,
                     hint_forwardrefs_class_basename=(
                         hint_forwardrefs_class_basename),
                 )
 
+                #FIXME: [SPEED] Optimize the following two string munging
+                #operations into a single string-munging operation resembling:
+                #    func_wrapper_code = CODE_RETURN_CHECK.format(
+                #        func_call_prefix=bear_call.func_wrapper_code_call_prefix,
+                #        check_expr=code_return_check_pith_unmemoized,
+                #    )
+                #
+                #Then define "CODE_RETURN_CHECK" in the "wrapsnip" submodule to
+                #resemble:
+                #    CODE_RETURN_CHECK = (
+                #        f'{CODE_RETURN_CHECK_PREFIX}{{check_expr}}'
+                #        f'{CODE_RETURN_CHECK_SUFFIX}'
+                #    )
+
                 # Python code snippet type-checking this return.
                 code_return_check_prefix = CODE_RETURN_CHECK_PREFIX.format(
                     func_call_prefix=bear_call.func_wrapper_code_call_prefix)
@@ -629,8 +667,7 @@ def _code_check_return(bear_call: BeartypeCall) -> str:
     except Exception as exception:
         reraise_exception_placeholder(
             exception=exception,
-            target_str=prefix_callable_decorated_return(
-                bear_call.func_wrappee),
+            target_str=prefix_beartypeable_return(bear_call.func_wrappee),
         )
 
     # Return this code.
@@ -645,13 +682,13 @@ def _unmemoize_func_wrapper_code(
 ) -> str:
     '''
     Convert the passed memoized code snippet type-checking any parameter or
-    return of the decorated callable into a memoized code snippet type-checking
-    a specific parameter or return of that callable.
+    return of the decorated callable into an "unmemoized" code snippet
+    type-checking a specific parameter or return of that callable.
 
     Specifically, this function (in order):
 
     #. Globally replaces all references to the
-       :data:`CODE_PITH_ROOT_PARAM_NAME_PLACEHOLDER` placeholder substring
+       :data:`.CODE_PITH_ROOT_PARAM_NAME_PLACEHOLDER` placeholder substring
        cached into this code with the passed ``pith_repr`` parameter.
     #. Unmemoizes this code by globally replacing all relative forward
        reference placeholder substrings cached into this code with Python
