@@ -14,6 +14,7 @@ This private submodule is *not* intended for importation by downstream callers.
 
 # ....................{ IMPORTS                            }....................
 from beartype.roar import BeartypeDecorHintPep484585Exception
+from beartype._data.datakind import ARG_NAME_RETURN
 from beartype._data.datatyping import TypeException
 from beartype._data.hint.pep.sign.datapepsigns import HintSignCoroutine
 from beartype._data.hint.pep.sign.datapepsignset import (
@@ -26,11 +27,7 @@ from beartype._util.func.utilfunctest import (
     is_func_async_generator,
     is_func_sync_generator,
 )
-from beartype._util.hint.pep.proposal.pep484585.utilpep484585arg import (
-    get_hint_pep484585_args_3)
-from beartype._util.hint.pep.utilpepget import get_hint_pep_sign_or_none
-from beartype._util.text.utiltextlabel import (
-    prefix_callable_decorated_return)
+from beartype._util.text.utiltextprefix import prefix_beartypeable_return
 from collections.abc import (
     AsyncGenerator,
     Callable,
@@ -69,9 +66,14 @@ def reduce_hint_pep484585_func_return(
           by a sign in the :data:`HINT_SIGNS_RETURN_GENERATOR_ASYNC` set.
     '''
 
+    # Avoid circular import dependencies.
+    from beartype._util.hint.pep.proposal.pep484585.utilpep484585arg import (
+        get_hint_pep484585_args_3)
+    from beartype._util.hint.pep.utilpepget import get_hint_pep_sign_or_none
+
     # Type hint annotating this callable's return, which the caller has already
     # explicitly guaranteed to exist.
-    hint = func.__annotations__['return']
+    hint = func.__annotations__[ARG_NAME_RETURN]
 
     # Sign uniquely identifying this hint if any *OR* "None" otherwise (e.g.,
     # if this hint is an isinstanceable class).
@@ -81,6 +83,7 @@ def reduce_hint_pep484585_func_return(
     if is_func_coro(func):
         # If this hint is "Coroutine[...]"...
         if hint_sign is HintSignCoroutine:
+
             # 3-tuple of all child type hints subscripting this hint if
             # subscripted by three such hints *OR* raise an exception.
             hint_args = get_hint_pep484585_args_3(
@@ -205,10 +208,10 @@ def _die_of_hint_return_invalid(
 
     # Type hint annotating this callable's return, which the caller has already
     # explicitly guaranteed to exist.
-    hint = func.__annotations__['return']
+    hint = func.__annotations__[ARG_NAME_RETURN]
 
     # Raise an exception of this type with a message suffixed by this suffix.
     raise exception_cls(
-        f'{prefix_callable_decorated_return(func)}type hint '
+        f'{prefix_beartypeable_return(func)}type hint '
         f'{repr(hint)} contextually invalid{exception_suffix}'
     )
