@@ -4,13 +4,14 @@
 # See "LICENSE" for further details.
 
 '''
-Project-wide **EP-compliant type hint tester utilities** (i.e., callables
-validating arbitrary objects to be PEP-compliant type hints).
+Project-wide **PEP-compliant type hint tester** (i.e., callable validating an
+arbitrary object to be a PEP-compliant type hint) utilities.
 
 This private submodule is *not* intended for importation by downstream callers.
 '''
 
 # ....................{ IMPORTS                            }....................
+from beartype.meta import URL_PEP585_DEPRECATIONS
 from beartype.roar import (
     BeartypeDecorHintPepException,
     BeartypeDecorHintPepUnsupportedException,
@@ -205,7 +206,7 @@ def die_if_hint_pep_unsupported(
     BeartypeDecorHintPep484Exception
         If this object is the PEP-compliant :attr:`typing.NoReturn` type hint,
         which is contextually valid in only a single use case and thus
-        supported externally by the :mod:`beartype._decor._wrapper.wrappermain`
+        supported externally by the :mod:`beartype._decor._wrap.wrapmain`
         submodule rather than with general-purpose automation.
     '''
 
@@ -230,8 +231,9 @@ def die_if_hint_pep_unsupported(
     # *ONLY* as a return annotation, raise an exception specific to this hint.
     if hint is NoReturn:
         raise BeartypeDecorHintPep484Exception(
-            f'{exception_prefix}return type hint "{repr(hint)}" invalid (i.e., '
-            f'"typing.NoReturn" valid only as non-nested return annotation).'
+            f'{exception_prefix}PEP 484 type hint "{repr(hint)}" '
+            f'invalid in this type hint context (i.e., '
+            f'"{repr(hint)}" valid only as non-nested return annotation).'
         )
     # Else, this is any PEP-compliant type hint other than "typing.NoReturn".
 
@@ -344,12 +346,12 @@ def warn_if_hint_pep_deprecated(
         #See also: https://docs.python.org/3/library/warnings.html#warnings.warn
         warn(
             (
-                f'PEP 484 type hint {repr(hint)} deprecated by PEP 585 '
-                f'scheduled for removal in the first Python version '
-                f'released after October 5th, 2025. To resolve this, import '
-                f'this hint from "beartype.typing" rather than "typing". '
-                f'See this discussion for further details and alternatives:\n'
-                f'    https://github.com/beartype/beartype#pep-585-deprecations'
+                f'PEP 484 type hint {repr(hint)} deprecated by PEP 585. '
+                f'This hint is scheduled for removal in the first Python '
+                f'version released after October 5th, 2025. To resolve this, '
+                f'import this hint from "beartype.typing" rather than "typing". '
+                f'For further commentary and alternatives, see also:\n'
+                f'    {URL_PEP585_DEPRECATIONS}'
             ),
             BeartypeDecorHintPep585DeprecationWarning,
         )
