@@ -11,6 +11,24 @@
 #   List of all options supported in this file.
 
 # ....................{ TODO                               }....................
+#FIXME: [CODE] Politely suggest that "pydata-sphinx-theme' incorporate
+#functionality for wrapping long function signatures. Thankfully, this has
+#already been (possibly elegantly) achieved by the competing Sphinx-Immaterial
+#theme via CSS governed by well-documented "conf.py" globals. See also:
+#    https://sphinx-immaterial.readthedocs.io/en/stable/apidoc/format_signatures.html
+#FIXME: [CODE] Thankfully, somebody's already implemented a crude CSS hack
+#enabling this for Python signatures. Let's incorporate this when time admits:
+#    https://github.com/sphinx-doc/sphinx/issues/1514#issuecomment-742703082
+
+#FIXME: [THEME] Consider generalizing our theme from the somewhat low-level
+#"pydata-sphinx-theme" theme to the substantially higher-level
+#"sphinx-book-theme". The latter is built in terms of the former, but layers on
+#additional functionality for interactive Jupyter BinderHub-based cell content.
+#That would be just fantastic, because we have so *UTTERLY* much "code-block"
+#content distributed across our documentation; rendering that as interactive
+#content that users could actually run would be a *HUGE* win. See also:
+#    https://github.com/executablebooks/sphinx-book-theme
+
 #FIXME: [EXTENSION] Add "sphinx-notfound-page" support to enable us to provide
 #a sane 404 page for non-existent pages. In fact, we already appear to have a
 #"404.rst" document. Well, isn't that noice. Because we can't be bothered to
@@ -40,9 +58,9 @@ from pathlib import Path
 #     https://stackoverflow.com/a/44569249/2809027
 # Dismantled, this is:
 #
-# * ".resolve(...)", creating and returning a new high-level "Path"
-#   object canonicalizing the relative dirname with which the original "Path"
-#   object was instantiated into an absolute dirname.
+# * ".resolve(...)", creating and returning a new high-level "Path" object
+#   canonicalizing the relative dirname with which the original "Path" object
+#   was instantiated into an absolute dirname.
 # * "strict=True", raising a "FileNotFoundError" if this directory does *NOT*
 #   exist.
 _DOC_SRC_DIR = Path(__file__).parent.resolve(strict=True)
@@ -68,6 +86,7 @@ sys.path.insert(0, str(_ROOT_DIR))
 # * https://protips.readthedocs.io/git-tag-version.html
 #   "Inferring Release Number from Git Tags", detailing a clever one-liner
 #   harvesting this specifier from the most recent git tag.
+# from beartype import beartype
 from beartype.meta import (
     AUTHORS,
     COPYRIGHT,
@@ -75,6 +94,7 @@ from beartype.meta import (
     SPHINX_THEME_NAME,
     VERSION,
 )
+from beartype.typing import Optional
 from beartype._util.mod.utilmodtest import is_module
 # from warnings import warn
 
@@ -106,8 +126,7 @@ version = VERSION
 suppress_warnings = [
     #FIXME: *THIS IS TERRIBLE.* Generally speaking, we do want Sphinx to inform
     #us about cross-referencing failures. Remove this hack entirely after Sphinx
-    #resolves this open issue:
-    #    https://github.com/sphinx-doc/sphinx/issues/4961
+    #resolves this open issue: https://github.com/sphinx-doc/sphinx/issues/4961
     # Squelch mostly ignorable warnings resembling:
     #     WARNING: more than one target found for cross-reference 'TypeHint':
     #     beartype.door._doorcls.TypeHint, beartype.door.TypeHint
@@ -129,13 +148,26 @@ templates_path = ['_templates']
 # List of pathname patterns relative to this "doc/src/" subdirectory matching
 # all files and directories to be ignored when finding source files. Note this
 # list also affects the "html_static_path" and "html_extra_path" settings.
+#
+# Note that this global obsoletes the prior "unused_doc", which served a similar
+# goal -- albeit in a less general-purpose manner.
 exclude_patterns = [
     # Ignore our URI repository, which magical logic below dynamically includes
     # on all pages through the classic "rst_epilog" trick.
-    _LINKS_FILENAME,
+    '_links.rst',
 ]
 
 # ....................{ SETTINGS ~ rst                     }....................
+# String of arbitrary reStructuredText (reST) to be implicitly prepended to the
+# contents of *ALL* reST documents rendered by this configuration, initialized
+# to the empty string for safety.
+rst_prolog = '''
+.. tip::
+
+   `Feed the bear <GitHub Sponsors_>`__! `Animals wearing jewelry
+   <beadspace9_>`__! *What is even going on with this banner!?*
+'''
+
 # String of arbitrary reStructuredText (reST) to be implicitly appended to the
 # contents of *ALL* reST documents rendered by this configuration, initialized
 # to the empty string for safety.
@@ -148,6 +180,7 @@ rst_epilog = ''
 #     https://stackoverflow.com/a/61694897/2809027
 with open(_LINKS_FILENAME, encoding='utf-8') as links_file:
     rst_epilog += links_file.read()
+# print(f'rst_epilog: {rst_epilog}')
 
 # ....................{ EXTENSIONS ~ mandatory             }....................
 # Add any Sphinx extension module names here, as strings. They can be
@@ -185,39 +218,6 @@ extensions = [
     # those modules to these listings.
     'sphinx.ext.viewcode',
 
-    # ..................{ BUILTIN ~ autodoc                  }..................
-    #FIXME: We currently prefer "autoapi" (which avoids runtime imports and code
-    #execution) to "autodoc" (which performs runtime imports and code
-    #execution), but nonetheless preserve this for posterity.
-    #FIXME: Almost certainly insufficient. Since this is Sphinx, the
-    #"autodoc" extension requires additional non-trivial configuration. Note,
-    #however, that the "autosummary" extension enabled below *SHOULD* automate
-    #most of that configuration once its fully working. Ergo, avoid investing a
-    #considerable amount of effort in "autodoc" itself. Try "autosummary"!
-
-    # Builtin extension autogenerating reStructuredText documentation from
-    # class, callable, and variable docstrings embedded in Python modules,
-    # documenting this project's public (and optionally also private) API.
-    # 'sphinx.ext.autodoc',
-
-    #FIXME: How does this compare with "viewcode"? Are the two at all
-    #comparable? Does merely one suffice? Research us up, please.
-    #FIXME: Almost certainly insufficient. Since this is Sphinx, the
-    #"autosummary" extension requires additional non-trivial configuration that
-    #absolutely *SHOULD* come bundled with Sphinx but currently isn't for
-    #nebulous reasons. See this StackOverflow answer for an exhaustive solution
-    #requiring copying of third-party templates into our configuration:
-    #    https://stackoverflow.com/a/62613202/2809027
-    #Lastly, note that this issue may have been resolved by a bleeding-edge
-    #Sphinx version by the time we finally resolve this. See also the end of:
-    #    https://github.com/sphinx-doc/sphinx/issues/7912
-
-    # # Builtin extension autosummarizing reStructuredText documentation
-    # # autogenerated by the "autodoc" extension (listed above). By default,
-    # # the "autodoc" extension fails to automatically structure (i.e., summarize
-    # # with a single compact HTML markup tag) the documentation it generates.
-    # 'sphinx.ext.autosummary',
-
     # ..................{ THIRD-PARTY                        }..................
     # Third-party Sphinx extensions required to be externally installed. For
     # usability, this block should typically be empty. Third-party Sphinx
@@ -239,29 +239,41 @@ extensions = [
     #   module-scoped attributes due to long-standing Sphinx limitations.
     #   Conversely, "autoapi" generates usable TOC entries out-of-the-box with
     #   *NO* configuration or kludges required.
-    'autoapi.extension',
+
+    #FIXME: Temporarily disabled in a pell-mell rush to host our "README.rst"
+    #file on RTD. Once we finish that laborious process, let's revisite
+    #"autoapi". Doing so will prove non-trivial, as "autoapi" currently emits
+    #~300 warnings -- some of which are ignorable, but most of which are not.
+    # 'autoapi.extension',
 ]
 
 # ....................{ EXTENSIONS ~ optional              }....................
 # Third-party Sphinx extensions conditionally used if externally installed.
 
-# If the third-party Sphinx extension defining the custom HTML theme preferred
-# by this documentation is importable under the active Python interpreter,
-# enable this theme for substantially improved rendering.
-if is_module(SPHINX_THEME_NAME):
-    # Set the HTML theme to this theme.
+# ....................{ EXTENSIONS ~ optional : theme      }....................
+# Fully-qualified name of the package providing the third-party Sphinx extension
+# defining the custom HTML theme preferred by this documentation, globally
+# substituting hyphens with underscores to produce a valid Python identifier.
+_SPHINX_THEME_MODULE_NAME = SPHINX_THEME_NAME.replace('-', '_')
+
+# If this package is importable under the active Python interpreter, enable this
+# theme for improved HTML rendering.
+if is_module(_SPHINX_THEME_MODULE_NAME):
+    # Set the HTML theme to this package.
     #
-    # Note that we intentionally do *NOT* do this, which other themes require:
-    #     Register the fully-qualified name of this extension.
+    # Note that we do *NOT* do this, which non-theme extensions require:
+    #     # Register the fully-qualified name of this extension.
     #     extensions.append(SPHINX_THEME_NAME)
-    # Why? Because doing so induces this exception from Furo:
+    #
+    # Why? Because doing so induces this exception from modern themes like Furo
+    # and PyData:
     #     Extension error (furo):
     #     Handler <function _builder_inited at 0x7f9be7bf2040> for event
     #     'builder-inited' threw an exception (exception: Did you list 'furo' in
     #     the `extensions` in conf.py? If so, please remove it. Furo does not
     #     work with non-HTML builders and specifying it as an `html_theme` is
     #     sufficient.)
-    html_theme = SPHINX_THEME_NAME
+    html_theme = _SPHINX_THEME_MODULE_NAME
 # Else, this theme is unavailable. In this case, fallback to Sphinx's default
 # HTML theme *AND*...
 else:
@@ -273,12 +285,74 @@ else:
     # Emit a non-fatal warning informing end users of this fallback.
     print(
         (
-            'WARNING: '
-            f'Optional Sphinx extension "{SPHINX_THEME_NAME}" not found; '
-            'falling back to default Sphinx HTML theme.'
+            f'WARNING: Optional Sphinx extension "{SPHINX_THEME_NAME}" '
+            f'not found; falling back to default Sphinx HTML theme.'
         ),
     )
 
+# ....................{ EXTENSIONS ~ optional : non-theme  }....................
+#FIXME: Restore @beartype once working. For unknown reasons, ReadTheDocs (RTD)
+#fails with non-human-readable exceptions *REMOTELY* when we attempt to decorate
+#a function defined in this file by @beartype. We have tried in vain to
+#replicate this locally (e.g., in our test_sphinx_docs_other() functional test).
+#Until we can, attempting to resolve this is mostly an exercise in futility.
+#FIXME: Actually, that test does now replicate this issue. Let's investigate
+#further via that test when time permits, please.
+#@beartype
+def _register_extension_or_warn(
+    # Mandatory parameters.
+    extension_name: str,
+
+    # Optional parameters.
+    warning_message: Optional[str] = None,
+) -> None:
+    '''
+    Register the extension with the passed package name if that package is
+    importable under the active Python interpreter *or* print a non-fatal
+    warning otherwise (i.e., if that package is unimportable).
+
+    Parameters
+    ----------
+    extension_name : str
+        Fully-qualified name of the package providing this extension.
+    warning_message : Optional[str]
+        Human-readable message to be printed when this package is unimportable.
+        Defaults to ``None``, in which case a standard message is printed.
+    '''
+
+    # If this package is importable under the active Python interpreter, append
+    # the name of this package to the list of all Sphinx extensions to enable.
+    if is_module(extension_name):
+        extensions.append(extension_name)
+    # Else, this package is unimportable. In this case, print the passed
+    # non-fatal warning message.
+    else:
+        # Substring unconditionally prefixing this warning message.
+        WARNING_MESSAGE_PREFIX = (
+            f'WARNING: Optional Sphinx extension "{extension_name}" not found.')
+
+        # Replace this warning message with either...
+        warning_message = (
+            # If the caller passed a warning message, this message prefixed by
+            # this substring;
+            f'{WARNING_MESSAGE_PREFIX} {warning_message}'
+            # Else, this prefix as is.
+            if warning_message else
+            WARNING_MESSAGE_PREFIX
+        )
+
+        # Print this warning message.
+        print(warning_message)
+
+
+#FIXME: Uncomment *AFTER* this extension resolves this currently open issue:
+#    https://github.com/wpilibsuite/sphinxext-opengraph/issues/98
+# # Register the third-party "sphinxext-opengraph" extension if available. This
+# # extension autogenerates Open Graph metadata describing this documentation.
+# # Social media and search engine giants commonly consume this metadata to
+# # optimize both the ranking and presentation of links to this documentation.
+# _register_extension_or_warn('sphinxext.opengraph')
+
 # ....................{ EXTENSIONS ~ autodoc               }....................
 # "autoapi.extension"-specific settings. See also:
 #     https://sphinx-autoapi.readthedocs.io/en/latest/reference/config.html
@@ -403,6 +477,16 @@ napoleon_custom_sections = [
 # pygments_style = "autumn"
 # pygments_dark_style = "monokai"
 
+# ....................{ LANG ~ python                      }....................
+# Wrap Python callable and class signatures exceeding this maximum number of
+# plaintext characters such that each parameter of those signatures is then
+# delegated its own discrete line.
+#
+# Note that this is currently only a prospective PR likely to be merged and is
+# thus merely something we're currently excited about. See also:
+#    https://github.com/sphinx-doc/sphinx/pull/11011
+python_maximum_signature_line_length = 80
+
 # ....................{ BUILD ~ html                       }....................
 #FIXME: Define the "html_favicon" setting as well -- once we actually create a
 #favicon, of course. *sigh*
@@ -416,6 +500,113 @@ html_logo = 'https://raw.githubusercontent.com/beartype/beartype-assets/main/bad
 # so a file named "default.css" will overwrite the builtin "default.css".
 html_static_path = ['_static']
 
+# Dictionary mapping from the names of theme-specific options to those options.
+html_context = {
+    # ....................{ PYDATA                         }....................
+    # HTML options specific to the "pydata-sphinx-theme" theme.
+
+    # Default the current theme to dark rather than light. Embrace the darkness!
+    'default_mode': 'dark',
+
+    # ....................{ PYDATA ~ edit                  }....................
+    # HTML options supporting the "use_edit_page_button" setting enabled below.
+
+    'github_user': 'beartype',
+    'github_repo': 'beartype',
+    'github_version': 'main',
+    'doc_path': 'doc/src',
+}
+
+# Dictionary mapping from the names of theme-specific options to those options.
+# We have *NO* idea why Sphinx supports both this and the parallel
+# "html_context" dictionary -- but it does. (Just pretend this isn't happening.)
+html_theme_options = {
+    # ....................{ PYDATA                         }....................
+    # HTML options specific to the "pydata-sphinx-theme" theme.
+
+    #FIXME: Add favicon support here, please. See also:
+    #    https://pydata-sphinx-theme.readthedocs.io/en/v0.7.2/user_guide/configuring.html#adding-favicons
+
+    # Announcement banner defined as a string of arbitrary HTML, temporarily
+    # displayed at the top of each page until the user begins scrolling.
+    'announcement': (
+        '<p>'
+        '<a href="https://github.com/sponsors/leycec">Feed the bear</a>! '
+        '<a href="https://beadspace9.ca">Animals wearing jewelry</a>! '
+        '<i>What is even going on with this banner!?</i>'
+        '</p>'
+    ),
+
+    # List of one or more icon link descriptions. See also:
+    # * Official theme-specific documentation for this setting:
+    #   https://pydata-sphinx-theme.readthedocs.io/en/v0.7.2/user_guide/configuring.html#configure-icon-links
+    # * Official search engine for icons published by the third-party
+    #   "FontAwesome 5 Free" project and supported by this setting:
+    #   https://fontawesome.com/icons?d=gallery&m=free
+    #
+    # Note that:
+    # * This theme requires long-form FontAwesome styles (e.g., "fa-brand",
+    #   "fa-solid") to be abbreviated to these three-letter abbreviations:
+    #   * "fa-brand" -> "fab".
+    #   * "fa-regular" -> "far".
+    #   * "fa-solid" -> "fas".
+    'icon_links': [
+        {
+            'name': 'GitHub',
+            'url': 'https://github.com/beartype/beartype',
+            'icon': 'fab fa-github-square',
+        },
+        {
+            'name': 'Libraries.io',
+            'url': 'https://libraries.io/pypi/beartype',
+            'icon': 'fas fa-chart-area',
+        },
+        {
+            'name': 'PyPI',
+            'url': 'https://pypi.org/project/beartype',
+            'icon': 'fab fa-python',
+        },
+        {
+            'name': 'Anaconda',
+            'url': 'https://anaconda.org/conda-forge/beartype',
+            'icon': 'far fa-circle',
+        },
+        {
+            'name': 'ReadTheDocs',
+            'url': 'https://readthedocs.org/projects/beartype',
+            'icon': 'fas fa-book',
+        },
+    ],
+
+    # Add an "Edit this Page" button to the secondary sidebar of each page,
+    # enabling users to trivially submit an automated pull request (PR) politely
+    # requesting modifications to the contents of that page.
+    #
+    # Note that enabling this also requires adding various URIs and paths to
+    # the "html_context" global describing the remote host hosting this git
+    # repository (e.g., GitHub).
+    #
+    # See also upstream documentation on this subject:
+    #     https://pydata-sphinx-theme.readthedocs.io/en/latest/user_guide/source-buttons.html
+    'use_edit_page_button': True,
+}
+
+#FIXME: This setting is currently undocumented but appears to internally govern
+#the "n_links_before_dropdown" parameter passed to the
+#generate_header_nav_html() function. Submit an issue requesting documentation!
+#FIXME: As expected, this is ignored. Until resolved, let's try temporarily
+#resolving this on our end by overriding the "navbar-nav.html" template. See:
+#    https://github.com/pydata/pydata-sphinx-theme/blob/157c9ab2c93e755141e48d13b5f193c6169d9dd8/src/pydata_sphinx_theme/theme/pydata_sphinx_theme/components/navbar-nav.html#L8
+#
+#Issue, please!
+
+# Increase the maximum number of links displayed by "pydata-sphinx-theme" in the
+# top-most navigation bar. "pydata-sphinx-theme" displays all remaining links in
+# a dropdown entitled "More" that, when clicked, vertically lists those links.
+# "pydata-sphinx-theme" defaults this global to 5, which is insufficient for the
+# larger number of links required for this documentation.
+theme_header_links_before_dropdown = 10
+
 # ....................{ BUILD ~ html : mathjax             }....................
 # URL to the top-level "MathJax.js" script providing MathJax. If unspecified,
 # the user *MUST* locally install MathJax. Note that MathJax is locally
